from __future__ import print_function

from botleague_helpers.config import blconfig
from botleague_helpers.config import get_test_name_from_callstack


class SimpleKeyValueStore:
    collection_name: str = 'simple_key_value_store'

    def get(self, key):
        raise NotImplementedError()

    def set(self, key, value):
        raise NotImplementedError()


class SimpleKeyValueStoreFirestore(SimpleKeyValueStore):
    def __init__(self):
        from firebase_admin import firestore
        blconfig.ensure_firebase_initialized()
        self.kv = firestore.client().collection(self.collection_name)

    def get(self, key):
        value = self.kv.document(key).get().to_dict() or {}
        if key in value and len(value) == 1:
            # Document just contains the one field with the same name
            # as the document, so just return the value of the field.
            value = value[key]
        return value

    def set(self, key, value):
        if not isinstance(value, dict):
            # Value is just a field
            self.kv.document(key).set({key: value})
        else:
            # Value is a collection
            self.kv.document(key).set(value)


class SimpleKeyValueStoreLocal(SimpleKeyValueStore):
    def __init__(self):
        self.kv = {}

    def get(self, key):
        return self.kv[key]

    def set(self, key, value):
        self.kv[key] = value


def get_key_value_store() -> SimpleKeyValueStore:
    test_name = get_test_name_from_callstack()
    if test_name:
        print('We are in a test, %s, so not using Firestore' % test_name)
        return SimpleKeyValueStoreLocal()
    elif blconfig.should_use_firestore:
        print('Using Firestore backed key value store')
        return SimpleKeyValueStoreFirestore()
    else:
        print('SHOULD_USE_FIRESTORE is false, so not using Firestore')
        return SimpleKeyValueStoreLocal()



import inspect
import os

import github


class Config:
    # Constants
    should_use_firestore = os.environ.get(
        'SHOULD_USE_FIRESTORE', 'true') == 'true'
    is_test = 'IS_TEST' in os.environ
    should_gen_key = 'should_gen_leaderboard'
    token_name = 'LEADERBOARD_GITHUB_TOKEN'

    # Properties that will change during tests
    # --------------------------------------------------------------------------
    _github_token: str = None
    _firebase_initialized: bool = False

    @property
    def github_token(self) -> str:
        test_name = get_test_name_from_callstack()
        if test_name:
            print('In test %s so returning "" for github token' % test_name)
            self.is_test = True
            ret = ''
        elif self.is_test:
            print('IS_TEST is set, so returning "" for github token')
            ret = ''
        elif self.token_name in os.environ:
            # We're not in a test and have not set the token, set from env
            print('Found %s in environment' % self.token_name)
            ret = os.environ[self.token_name]
        elif not self.should_use_firestore:
            # We're not in a test, but the env has dictated not to use Firestore
            print('SHOULD_USE_FIRESTORE is false, so returning "" '
                  'for github token')
            ret = ''
        elif self._github_token is None:
            # We're not in a test and have not set the token, fetch from
            # Firestore
            self.ensure_firebase_initialized()
            from firebase_admin import firestore
            print('Obtaining secrets from Firestore...')
            secrets = firestore.client().collection('secrets')
            ret = secrets.document(self.token_name).get().\
                to_dict()['token']
        else:
            # We're not in a test and have already set the token
            ret = self._github_token
        self._github_token = ret
        return ret

    def ensure_firebase_initialized(self):
        if not self._firebase_initialized:
            import firebase_admin

            try:
                firebase_admin.initialize_app()
            except Exception as e:
                raise RuntimeError(
                    'Could not initialize firestore, '
                    'set SHOULD_USE_FIRESTORE=false'
                    ' locally to use temp storage.')
            self._firebase_initialized = True


def get_test_name_from_callstack() -> str:
    ret = ''
    for level in inspect.stack():
        fn = level.function
        test_prefix = 'test_'
        if fn.startswith(test_prefix):
            test_name = fn[len(test_prefix):]
            return test_name
    return ret


if 'GITHUB_DEBUG' in os.environ:
    github.enable_console_debug_logging()


blconfig = Config()


def activate_test_mode():
    blconfig.is_test = True
    disable_firestore_access()


def disable_firestore_access():
    blconfig.should_use_firestore = False



import os
from os.path import abspath, dirname, join
from typing import List

from setuptools import setup


def get_requires() -> List[str]:
    with open('requirements.txt') as reqs_file:
        reqs = reqs_file.read().split(os.sep)
    return reqs


# Read the README markdown data from README.md
with open(abspath(join(dirname(__file__), 'README.md')), 'rb') as readme_file:
    __readme__ = readme_file.read().decode('utf-8')

setup(
    name='botleague-helpers',
    version='0.0.5',
    description='Helpers for Bot League',
    long_description=__readme__,
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Environment :: Console'
    ],
    keywords='botleague',
    url='http://github.com/deepdrive/botleague-helpers',
    author='Craig Quiter',
    author_email='craig@deepdrive.io',
    license='MIT',
    packages=['botleague_helpers'],
    zip_safe=True,
    python_requires='>=3.7',
    install_requires=get_requires()
)



import inspect
import os
from os.path import join

from botleague_helpers.constants import SHOULD_USE_FIRESTORE

import constants as c
from tests.test_constants import DATA_DIR
from util import get_str_or_json, read_file


class Mockable:
    test_name: str

    def __init__(self):
        self.test_name = get_test_name_from_callstack()
        if not self.test_name:
            raise RuntimeError('Did not find a test method in the call'
                               ' stack!')

    def github_get(self, repo, filename):
        filepath = self.get_test_filename(filename)
        content_str = read_file(filepath)
        ret = get_str_or_json(content_str, filepath)
        return ret

    def get_test_filename(self, filename):
        return join(DATA_DIR, self.test_name, filename)

